"""..."""
from dataclasses import dataclass

from aligo.types import DatClass
from aligo.types.Enum import GetVideoPreviewCategory, VideoTemplateID


@dataclass
class GetVideoPreviewPlayInfoRequest(DatClass):
    """..."""
    file_id: str = None
    drive_id: str = None
    category: GetVideoPreviewCategory = 'live_transcoding'
    template_id: VideoTemplateID = None
    url_expire_sec: int = 14400



"""..."""
from dataclasses import dataclass

from aligo.types import DatClass


@dataclass
class CancelShareLinkResponse(DatClass):
    """..."""
    # 这只是一个占位属性
    # 取消分享, 服务器返回为 '', 此对象仅用作与 Null对象 区分和统一结构和用法
    result: str = ''



"""..."""
from dataclasses import dataclass, field

from aligo.types import DatClass


@dataclass
class MoveFileResponse(DatClass):
    """..."""
    file_id: int = None
    drive_id: int = None
    domain_id: int = field(default=None, repr=False)
    async_task_id: str = field(default=None, repr=False)



from typing import Iterator

from aligo.core import BaseAligo
from aligo.core.Config import ADRIVE_V1_ALBUMHOME_ALBUMLIST, ADRIVE_V1_ALBUM_LIST_FILES, ADRIVE_V1_USER_ALBUMS_INFO
from aligo.request import AlbumListRequest, AlbumListFilesRequest
from aligo.response import AlbumInfoResponse, AlbumListResponse, ListResponse
from aligo.types import ListAlbumItem


class Album(BaseAligo):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._album_info = None

    @property
    def album_info(self) -> AlbumInfoResponse:
        if self._album_info is None:
            response = self.post(ADRIVE_V1_USER_ALBUMS_INFO)
            data = response.json()['data']
            self._album_info = AlbumInfoResponse(**data)
        return self._album_info

    def _core_list_album(self, body: AlbumListRequest) -> Iterator[ListAlbumItem]:
        yield from self._list_file(ADRIVE_V1_ALBUMHOME_ALBUMLIST, body, AlbumListResponse)

    def _core_list_album_files(self, body: AlbumListFilesRequest):
        yield from self._list_file(ADRIVE_V1_ALBUM_LIST_FILES, body, ListResponse)



"""..."""
from dataclasses import dataclass

from .Type import DatClass


@dataclass
class Privilege(DatClass):
    """..."""
    feature_id: str = None
    feature_attr_id: str = None
    quota: int = None



"""..."""
from dataclasses import dataclass
from typing import Dict, Union

from aligo.types import DatClass


@dataclass
class BatchSubRequest(DatClass):
    """..."""
    body: Union[DatClass, Dict]
    id: str
    url: str
    headers: Dict = None
    method: str = 'POST'

    def __post_init__(self):
        self.headers = {"Content-Type": "application/json"}
        super().__post_init__()



"""..."""
from dataclasses import dataclass, field
from typing import List

from aligo.types import DatClass
from aligo.types.Enum import CheckNameMode


@dataclass
class UpdateFileRequest(DatClass):
    """...."""
    name: str = None
    file_id: str = None
    drive_id: str = None
    check_name_mode: CheckNameMode = field(default='refuse', repr=False)
    custom_index_key: str = field(default=None, repr=False)
    description: str = field(default=None, repr=False)
    encrypt_mode: str = field(default=None, repr=False)
    hidden: str = field(default=None, repr=False)
    labels: List[str] = field(default=None, repr=False)
    meta: str = field(default=None, repr=False)
    starred: bool = field(default=None, repr=False)
    user_meta: str = field(default=None, repr=False)



"""..."""
from dataclasses import dataclass

from aligo.types import DatClass


@dataclass
class ArchiveStatusRequest(DatClass):
    """..."""
    file_id: str = None
    task_id: str = None
    drive_id: str = None
    domain_id: str = None



"""..."""
import hashlib
import os
import shutil
from datetime import datetime
from typing import Callable
from typing import Dict, Union
from typing import Optional

from aligo.core import Core
from aligo.request import GetFileListRequest, MoveFileToTrashRequest, CreateFolderRequest
from aligo.types import BaseFile


def utc_str_to_timestamp(utc: str) -> int:
    """'2022-04-16T11:07:03.276Z' -> timestamp"""
    return int(datetime.fromisoformat(utc[:-1]).timestamp()) + 28800


class SyncFolder(Core):
    """sync folder

    冲突：本地和云端冲突，一端是文件，另一端是文件夹，此种情况跳过，统一不处理

    三种同步模式：
        1. 双端同步
            差异文件保持最新；独有文件上传和下载，保持共有。

        2. 本地为主
            差异文件，本地为主；本地独有文件，上传；云端独有文件，忽略。

        3. 云端为主
            差异文件，云端为主；云端独有文件，下载；本地独有文件，忽略。
    """

    def sync_folder(
            self,
            local_folder: str,
            remote_folder: str,
            flag: Optional[bool] = None,
            file_filter: Callable[[Union[str, BaseFile]], bool] = lambda x: False,
            ignore_content: bool = False,
            follow_delete: bool = False,
            drive_id: str = None
    ):
        """
        sync folder
        :param local_folder: 本地文件夹路径
        :param remote_folder: 云端文件夹 file_id
        :param flag: 同步标志（类型），默认：None, 另外可选：True, False
            None：双端同步
            True：以本地为主
            False：以云端为主
        :param file_filter: 文件过滤函数，参数为 本地文件绝对路径 / 云端文件 BaseFile，返回值为 `True` 则过滤，可用于实现 只同步 特定文件 或 排除某些文件
        :param ignore_content: 是否忽略文件内容，默认：False
        :param follow_delete: 是否跟随删除，默认：False
        :param drive_id: 云端文件夹 drive_id
        :return:

        :Example:
        >>> from aligo import Aligo
        >>> ali = Aligo()
        >>> ali.sync_folder('<本地路径>', '<云端文件夹 file_id>')
        """
        if flag is None:
            self._auth.log.info('sync_folder: 双端同步')
        elif flag:
            self._auth.log.info('sync_folder: 本地为主')
        else:
            self._auth.log.info('sync_folder: 云端为主')

        if not os.path.exists(local_folder):
            self._auth.log.warning('本地文件夹不存在，创建: %s', local_folder)
            os.makedirs(local_folder)
        self.__sync_folder(local_folder, remote_folder, flag, file_filter, ignore_content, follow_delete, drive_id)

    def __sync_folder(
            self,
            local_folder: str,
            remote_folder: str,
            flag: Optional[bool],
            file_filter: Callable[[Union[str, BaseFile]], bool],
            ignore_content: bool,
            follow_delete: bool,
            drive_id: str):
        """..."""
        # 获取local_folder下的所有文件 # 转为 {文件名:文件路径} 字典模式
        local_files = {}
        for f in os.listdir(local_folder):
            local_file = os.path.join(local_folder, f)
            # 过滤文件只处理文件，跳过 文件夹
            if os.path.isfile(local_file) and file_filter(local_file):
                self._auth.log.debug(f'过滤本地文件 {local_file}')
                continue
            local_files[f] = local_file
        # 获取remote_folder下的所有文件 # 转为 {文件名:BaseFile对象} 字典模式
        remote_files: Dict[str, BaseFile] = {}
        for f in self._core_get_file_list(
                GetFileListRequest(remote_folder, drive_id=drive_id)
        ):
            remote_file = f.name
            if f.type == 'file' and file_filter(f):
                self._auth.log.debug(f'过滤云端文件 {remote_file}')
                continue
            remote_files[remote_file] = f

        if flag is None:
            self.__sync_all(
                drive_id, file_filter, flag, local_files, local_folder, remote_files, remote_folder, ignore_content,
                follow_delete)
        elif flag:
            self.__sync_local(
                drive_id, local_files, remote_files, remote_folder, flag, file_filter, local_folder, ignore_content,
                follow_delete)
        else:
            self.__sync_remote(drive_id, local_files, remote_files, local_folder, flag, file_filter, ignore_content,
                               follow_delete)

    def __sync_all(
            self, drive_id, file_filter, flag, local_files, local_folder, remote_files, remote_folder, ignore_content,
            follow_delete
    ):
        # 双端同步
        for f in list(local_files):
            # 在字典中获取并删除 local_file
            local_file = local_files.pop(f)
            # 文件夹则递归
            if os.path.isdir(local_file):
                # 判断云端是否存在
                if f in remote_files:
                    # 如果有，则判断是不是文件夹，不是就跳过
                    remote_file = remote_files.pop(f)
                    if remote_file.type == 'file':
                        # 跳过冲突
                        self._auth.log.warning(f'冲突：本地是文件夹，云端是文件，不处理 {f}')
                        continue
                else:
                    # 如果没有，则创建文件夹
                    self._auth.log.debug(f'本地是文件夹，云端不存在，创建云端文件夹，并递归 {f}')
                    remote_file = self._core_create_folder(
                        CreateFolderRequest(name=f, parent_file_id=remote_folder, drive_id=drive_id,
                                            check_name_mode='overwrite')
                    )
                self.__sync_folder(local_file, remote_file.file_id, flag, file_filter, ignore_content, follow_delete,
                                   drive_id)
                continue

            # 如果本地文件存在，且在云端也存在，则依次比较 size，sha1，时间戳
            if f in remote_files:
                # 在字典中获取并删除 remote_file
                remote_file = remote_files.pop(f)
                # 先判断文件类型
                if remote_file.type == 'folder':
                    # 没有主次端，不自动处理冲突
                    self._auth.log.warning(f'冲突：本地为文件，云端为文件夹，不处理 {f}')
                    continue

                # 获取 f 文件大小
                local_size = os.path.getsize(local_file)
                # 比较大小
                if local_size == remote_file.size:
                    # 跳过对比文件内容
                    if ignore_content:
                        self._auth.log.warning(f'忽略文件内容: 不处理 {f}')
                        continue

                    # 计算 f 文件 sha1
                    local_sha1 = self._core_sha1(local_file).lower()
                    # 如果sha1值相同，则跳过
                    if local_sha1 == remote_file.content_hash.lower():
                        self._auth.log.debug(f'文件 {f} 已存在，且sha1值相同，跳过')
                        continue
                # 否则对比时间戳，删除较旧文件，同步最新文件
                # 获取 f 文件的最后修改时间
                local_time = os.path.getmtime(local_file)
                local_time = int(local_time)
                # 获取云端文件的最后修改时间
                remote_time = remote_file.updated_at
                # 将UTC时间字符串转换为时间戳
                remote_time = utc_str_to_timestamp(remote_time)
                # 比较时间戳，删除较旧文件，同步最新文件
                if local_time > remote_time:
                    # 覆盖方式上传文件
                    self._auth.log.debug(f'本地文件较新，上传本地文件 {f}')
                    self.upload_file(file_path=local_file, parent_file_id=remote_folder, name=f,
                                     drive_id=drive_id, check_name_mode='overwrite')
                elif local_time < remote_time:
                    # 云端较新，删除本地，下载云端文件
                    self._auth.log.debug(f'云端较新，删除本地，下载云端文件 {f}')
                    os.remove(local_file)
                    self.download_files([remote_file], local_folder)
            else:
                # 不存在则直接上传
                self._auth.log.debug(f'云端不存在，上传本地文件 {f}')
                self.upload_file(file_path=local_file, parent_file_id=remote_folder, name=f,
                                 drive_id=drive_id, check_name_mode='overwrite')
        # 剩下的就是云端文件夹中有，本地文件夹中没有的文件，下载到本地
        if len(remote_files) != 0:
            self._auth.log.debug(f'本地不存在，下载云端文件 {len(remote_files)} {list(remote_files.keys())}')
            for remote_file in remote_files.values():
                if remote_file.type == 'file':
                    self.download_files([remote_file], local_folder)
                else:
                    self.download_folder(remote_file.file_id, local_folder)  # type: ignore

    def __sync_remote(self, drive_id, local_files, remote_files,
                      local_folder, flag, file_filter, ignore_content, follow_delete):
        # 以云端为主
        for f in list(remote_files):
            remote_file = remote_files.pop(f)
            # 文件夹则递归
            if remote_file.type == 'folder':
                # 判断本地是否存在
                if f in local_files:
                    # 如果有，则判断是否是文件夹，不是就忽略
                    local_file = local_files.pop(f)
                    if os.path.isfile(local_file):
                        # 跳过冲突
                        self._auth.log.warning(f'冲突：本地是文件，云端是文件夹，不处理 {f}')
                else:
                    # 如果没有，则创建本地文件夹
                    local_file = os.path.join(local_folder, f)
                    self._auth.log.debug(f'云端是文件夹，本地没有，创建文件夹，并递归 {local_file}')
                    if not os.path.exists(local_file):
                        os.mkdir(local_file)
                self.__sync_folder(local_file, remote_file.file_id, flag, file_filter, ignore_content, follow_delete,
                                   drive_id)
                continue

            # 如果云端文件存在，且在本地也存在，则依次比较 size，sha1，时间戳
            if f in local_files:
                local_file = local_files.pop(f)
                # 先判断文件类型
                if os.path.isdir(local_file):
                    # 云端为文件，本地却是文件夹，则递归删除文件夹，再下载文件
                    self._auth.log.debug('云端为文件，本地却是文件夹，则递归删除文件夹，再下载文件')
                    # 递归删除文件夹
                    shutil.rmtree(local_file)
                    # 下载文件
                    self.download_files([remote_file], local_folder)
                    continue

                # 获取 f 文件大小
                remote_size = remote_file.size
                # 比较大小
                if remote_size == os.path.getsize(local_file):
                    # 跳过对比文件内容
                    if ignore_content:
                        self._auth.log.warning(f'忽略文件内容: 不处理 {f}')
                        continue

                    # 计算 f 文件 sha1
                    local_sha1 = self._core_sha1(local_file).lower()
                    # 如果sha1值相同，则跳过
                    if local_sha1 == remote_file.content_hash.lower():
                        self._auth.log.debug(f'文件 {local_file} 已存在，且sha1值相同，跳过')
                        continue
                os.remove(local_file)
                self.download_files([remote_file], local_folder)
            else:
                # 不存在直接下载
                local_file = os.path.join(local_folder, f)
                self._auth.log.debug(f'本地不存在，下载云端文件 {local_file}')
                self.download_files([remote_file], local_folder)

        # 剩下的就是云端文件夹中没有，本地文件夹中有的文件，删除
        if follow_delete and len(local_files) != 0:
            self._auth.log.debug(f'云端不存在，本地存在 删除 {len(local_files)} {list(local_files.keys())}')
            for local_file in local_files.values():
                if os.path.isfile(local_file):
                    os.remove(local_file)
                else:
                    shutil.rmtree(local_file)

    def __sync_local(self, drive_id, local_files, remote_files,
                     remote_folder, flag, file_filter, local_folder, ignore_content, follow_delete):
        # 以本地为主
        for f in list(local_files):
            local_file = local_files.pop(f)
            # 文件夹则递归
            if os.path.isdir(local_file):
                # 判断云端是否存在
                if f in remote_files:
                    # 如果有，则判断是不是文件夹，不是就删除
                    remote_file = remote_files.pop(f)
                    if remote_file.type == 'file':
                        # 删除云端文件并创建文件夹
                        self._auth.log.warning(f'冲突：本地是文件夹，云端是文件，不处理 {f}')
                else:
                    # 如果没有，则创建文件夹
                    self._auth.log.debug(f'本地是文件夹，云端不存在，创建云端文件夹，并递归 {f}')
                    remote_file = self._core_create_folder(
                        CreateFolderRequest(name=f, parent_file_id=remote_folder, drive_id=drive_id,
                                            check_name_mode='overwrite')
                    )
                self.__sync_folder(local_file, remote_file.file_id, flag, file_filter, ignore_content, follow_delete,
                                   drive_id)
                continue

            # 如果本地文件存在，且在云端也存在，则依次比较 size，sha1，时间戳
            if f in remote_files:
                # 在字典中删除remote_file
                remote_file = remote_files.pop(f)
                # 先判断文件类型
                if remote_file.type == 'folder':
                    # 以本地文件夹为主，那只有删除云端文件夹，再上传本地文件
                    self._auth.log.debug(f'本地为文件，云端为文件夹，删除云端文件夹，并上传本地文件 {f}')
                    self._core_move_file_to_trash(
                        MoveFileToTrashRequest(file_id=remote_file.file_id, drive_id=drive_id)
                    )
                    self._auth.log.debug(f'上传本地文件 {f}')
                    self.upload_file(file_path=local_file, parent_file_id=remote_folder, name=f,
                                     drive_id=drive_id, check_name_mode='overwrite')
                    continue

                # 获取 f 文件大小
                local_size = os.path.getsize(local_file)
                # 比较大小
                if local_size == remote_file.size:
                    # 跳过对比文件内容
                    if ignore_content:
                        self._auth.log.warning(f'忽略文件内容: 不处理 {f}')
                        continue

                    # 计算 f 文件 sha1
                    local_sha1 = self._core_sha1(local_file).lower()
                    # 如果sha1值相同，则跳过
                    if local_sha1 == remote_file.content_hash.lower():
                        self._auth.log.debug(f'文件 {f} 已存在，且sha1值相同，跳过')
                        continue
                    else:
                        self.upload_file(file_path=local_file, parent_file_id=remote_folder, name=f,
                                         drive_id=drive_id, check_name_mode='overwrite')
                else:
                    self.upload_file(file_path=local_file, parent_file_id=remote_folder, name=f,
                                     drive_id=drive_id, check_name_mode='overwrite')
            else:
                # 不存在则直接上传
                self._auth.log.debug(f'云端不存在，上传本地文件 {f}')
                self.upload_file(file_path=local_file, parent_file_id=remote_folder, name=f,
                                 drive_id=drive_id, check_name_mode='overwrite')

        # 剩下的就是本地文件夹中没有，云端文件夹中有的文件，删除
        if follow_delete and len(remote_files) != 0:
            self._auth.log.debug(f'本地不存在，云端存在 删除 {len(remote_files)} {list(remote_files.keys())}')
            self.batch_move_to_trash([remote_file.file_id for remote_file in remote_files.values()])  # type: ignore

    @staticmethod
    def _core_sha1(param):
        """计算文件sha1"""
        sha1 = hashlib.sha1()
        with open(param, 'rb') as f:
            while True:
                data = f.read(8192)
                if not data:
                    break
                sha1.update(data)
        return sha1.hexdigest()



"""..."""
from dataclasses import dataclass

from aligo.types import DatClass, RateLimit
from aligo.types.Enum import BaseFileContentHashName


@dataclass
class GetDownloadUrlResponse(DatClass):
    """..."""
    expiration: str = None
    method: str = None
    size: int = None
    url: str = None
    cdn_url: str = None
    internal_url: str = None
    ratelimit: RateLimit = None
    crc64_hash: str = None
    content_hash: str = None
    content_hash_name: BaseFileContentHashName = None



"""..."""
from dataclasses import dataclass, field

from aligo.types import DatClass


@dataclass
class ShareFileSaveToDriveRequest(DatClass):
    """..."""
    share_id: str
    file_id: str
    to_parent_file_id: str = 'root'
    new_name: str = None
    auto_rename: bool = field(default=True, repr=False)
    overwrite: bool = field(default=False, repr=False)
    to_drive_id: str = None



"""..."""
from dataclasses import dataclass
from dataclasses import field
from typing import List

from .BaseFile import BaseFile
from .Type import DatClass


@dataclass
class ListAlbumItem(DatClass):
    """..."""
    name: str = None
    type: str = None
    album_id: int = None
    total_count: int = None
    file_list: List[BaseFile] = field(default_factory=list)



"""更新分享链接请求"""
from dataclasses import dataclass

from aligo.types import DatClass


@dataclass
class UpdateShareLinkRequest(DatClass):
    """
    更新分享链接请求

    Attributes:
        share_id (str):
        share_pwd (str): 提取码，0-64个字符。长度0表示没有提取码。
        expiration (str): 失效时间点。RFC3339格式，比如："2020-06-28T11:33:00.000+08:00"。永久有效: ""
        description (str): 描述
    """
    share_id: str
    share_pwd: str = None
    expiration: str = ''
    description: str = None



"""..."""
from dataclasses import dataclass

from .Type import DatClass


@dataclass
class EMailConfig(DatClass):
    """邮箱配置"""
    email: str  # 接收邮件的邮箱
    host: str  # 示例值 'smtp.163.com'
    port: int  # 示例值 465
    user: str  # 示例值 'aligo_notify@163.com'
    password: str  # 示例值 'IFMXTIOSDYZUMUYW'
    content: str = ''  # 附加内容



"""..."""
from typing import List, overload

from aligo.core import Core
from aligo.request import MoveFileRequest, BatchMoveFilesRequest
from aligo.response import MoveFileResponse, BatchSubResponse


class Move(Core):
    """..."""

    @overload
    def move_file(self, file_id: str,
                  to_parent_file_id: str = 'root',
                  new_name: str = None,
                  drive_id: str = None,
                  to_drive_id: str = None,
                  **kwargs) -> MoveFileResponse:
        """
        移动文件
        :param file_id: [必选] 文件ID
        :param to_parent_file_id: [可选] 目标文件夹ID
        :param new_name: [可选] 新文件名
        :param drive_id: [可选] 文件所在的网盘ID
        :param to_drive_id: [可选] 目标网盘ID
        :param kwargs: [可选] 其他参数
        :return: [MoveFileResponse]

        用法示例：
        >>> from aligo import Aligo, MoveFileRequest
        >>> ali = Aligo()
        >>> result = ali.move_file(file_id='<file_id>')  # 默认移动到 根目录
        >>> print(result)
        """

    @overload
    def move_file(self, body: MoveFileRequest) -> MoveFileResponse:
        """
        移动文件
        :param body: [必选] 移动文件请求体
        :return: [MoveFileResponse]

        用法示例：
        >>> from aligo import Aligo, MoveFileRequest
        >>> ali = Aligo()
        >>> body = MoveFileRequest(file_id='<file_id>')  # 默认移动到 根目录
        >>> result = ali.move_file(body=body)
        >>> print(result)
        """

    def move_file(self, file_id: str = None,
                  to_parent_file_id: str = 'root',
                  new_name: str = None,
                  drive_id: str = None,
                  to_drive_id: str = None,
                  body: MoveFileRequest = None,
                  **kwargs) -> MoveFileResponse:
        """move_file"""
        if body is None:
            body = MoveFileRequest(
                file_id=file_id,
                drive_id=drive_id,
                to_drive_id=to_drive_id,
                to_parent_file_id=to_parent_file_id,
                new_name=new_name,
                **kwargs
            )
        return self._core_move_file(body)

    @overload
    def batch_move_files(self,
                         file_id_list: List[str],
                         to_parent_file_id: str = 'root',
                         drive_id: str = None,
                         **kwargs) -> List[BatchSubResponse[MoveFileResponse]]:
        """
        批量移动文件
        :param file_id_list: [必选] 文件ID列表
        :param to_parent_file_id: [可选] 目标文件夹ID
        :param drive_id: [可选] 文件所在的网盘ID
        :param kwargs: [可选] 其他参数
        :return: [List[BatchSubResponse]]

        用法示例：
        >>> from aligo import Aligo, BatchMoveFilesRequest
        >>> ali = Aligo()
        >>> file_id_list = ['<file1_id>', '<file2_id>']
        >>> result = ali.batch_move_files(file_id_list=file_id_list)  # 默认移动到 根目录
        >>> print(result[0].body.file_id)
        """

    @overload
    def batch_move_files(self, body: BatchMoveFilesRequest) -> List[BatchSubResponse[MoveFileResponse]]:
        """
        批量移动文件
        :param body: [必选] 批量移动文件请求体
        :return: [List[BatchSubResponse]]

        用法示例：
        >>> from aligo import Aligo, BatchMoveFilesRequest
        >>> ali = Aligo()
        >>> body = BatchMoveFilesRequest(file_id_list=['<file1_id>', '<file2_id>'])  # 默认移动到 根目录
        >>> result = ali.batch_move_files(body=body)
        >>> print(result[0].body.file_id)
        """

    def batch_move_files(self,
                         file_id_list: List[str] = None,
                         to_parent_file_id: str = 'root',
                         drive_id: str = None,
                         body: BatchMoveFilesRequest = None,
                         **kwargs) -> List[BatchSubResponse[MoveFileResponse]]:
        """batch_move_files"""
        if body is None:
            body = BatchMoveFilesRequest(
                drive_id=drive_id,
                file_id_list=file_id_list,
                to_parent_file_id=to_parent_file_id,
                **kwargs
            )
        result = self._core_batch_move_files(body)
        return list(result)



"""..."""
import os
from typing import List, overload, Callable

from aligo.core import Core
from aligo.error import AligoException
from aligo.request import GetFileRequest, GetFileListRequest, GetDownloadUrlRequest, BatchDownloadUrlRequest
from aligo.response import GetDownloadUrlResponse, BatchDownloadUrlResponse
from aligo.types import BaseFile


class Download(Core):
    """..."""

    def get_download_url(self,
                         file_id: str,
                         file_name: str = None,
                         expire_sec: int = 14400,
                         drive_id: str = None,
                         ) -> GetDownloadUrlResponse:
        """
        获取下载链接: 一般在已知 file_id 或 download_url 失效时使用
        :param file_id: [str] 文件 id
        :param file_name: Optional[str] 文件名
        :param expire_sec: Optional[int] 下载链接有效时间, 默认为 4 小时, 这也是允许的最大值
        :param drive_id: Optional[str] 文件所在的网盘 id
        :return: [GetDownloadUrlResponse]

        用法示例:
        >>> from aligo import Aligo
        >>> ali = Aligo()
        >>> result = ali.get_download_url(file_id='<file_id>')
        >>> print(result)
        """
        body = GetDownloadUrlRequest(
            file_id=file_id,
            drive_id=drive_id,
            file_name=file_name,
            expire_sec=expire_sec,
        )
        return self._core_get_download_url(body)

    def batch_download_url(self,
                           file_id_list: List[str],
                           expire_sec: int = 14400,
                           drive_id=None) -> List[BatchDownloadUrlResponse]:
        """
        批量获取下载链接
        :param file_id_list: [List[str]] 文件 id 列表
        :param expire_sec: [int] 下载链接有效时间, 默认为 4 小时, 这也是允许的最大值
        :param drive_id: [str] 文件所在的网盘 id
        :return: [List[BatchDownloadUrlResponse]]

        用法示例:
        >>> from aligo import Aligo
        >>> ali = Aligo()
        >>> result = ali.batch_download_url(file_id_list=['<file1_id>', '<file2_id>'])
        >>> print(result)
        """
        body = BatchDownloadUrlRequest(
            drive_id=drive_id,
            file_id_list=file_id_list,
            expire_sec=expire_sec
        )
        result = self._core_batch_download_url(body)
        return list(result)

    def download_folder(self, folder_file_id: str, local_folder: str = '.', drive_id: str = None,
                        file_filter: Callable[[BaseFile], bool] = lambda x: False) -> str:
        """
        下载文件夹
        :param folder_file_id: [str] 文件夹 id
        :param local_folder: [str] 本地文件夹路径, 默认为当前目录, 即下载到哪里
        :param drive_id: [str] 文件夹所在的网盘 id
        :param file_filter: 文件过滤函数
        :return: [str] 本地文件夹路径

        用法示例:
        >>> from aligo import Aligo
        >>> ali = Aligo()
        >>> result = ali.download_folder(folder_file_id='<folder_file_id>')
        >>> print(result)
        """
        if folder_file_id != 'root':
            folder = self._core_get_file(GetFileRequest(file_id=folder_file_id, drive_id=drive_id))
            local_folder = os.path.join(local_folder, self._del_special_symbol(folder.name))
        return self.__download_folder(folder_file_id, local_folder, drive_id, file_filter=file_filter)

    def __download_folder(self, folder_file_id: str, local_folder: str = '.', drive_id: str = None,
                          file_filter: Callable[[BaseFile], bool] = lambda x: False) -> str:
        """下载文件夹"""
        # 创建文件夹, 即使文件夹为空
        os.makedirs(local_folder, exist_ok=True)
        files = []
        for file in self._core_get_file_list(GetFileListRequest(parent_file_id=folder_file_id, drive_id=drive_id)):
            if file_filter(file):
                continue
            if file.type == 'folder':
                self.__download_folder(folder_file_id=file.file_id,
                                       local_folder=os.path.join(local_folder, self._del_special_symbol(file.name)))
                continue
            files.append(file)
        self.download_files(files, local_folder=local_folder)
        return os.path.abspath(local_folder)

    @overload
    def download_file(self, *, file_path: str, url: str) -> str:
        """
        根据下载地址下载文件
        :param file_path: [str] 文件路径
        :param url: [str] 下载地址
        :return: [str] 本地文件路径

        用法示例:
        >>> from aligo import Aligo
        >>> ali = Aligo()
        >>> result = ali.download_file(file_path='<file_path>', url='<url>')
        >>> print(result)
        """

    @overload
    def download_file(self, *, file_id: str, local_folder: str = '.') -> str:
        """
        根据 file_id 下载文件
        :param file_id: [str] 文件 id
        :param local_folder: Optional[str] 本地文件夹路径, 默认为当前目录, 即下载到哪里
        :return: [str] 本地文件路径

        用法示例:
        >>> from aligo import Aligo
        >>> ali = Aligo()
        >>> result = ali.download_file(file_id='<file_id>')
        >>> print(result)
        """

    @overload
    def download_file(self, *, file: BaseFile, local_folder: str = '.') -> str:
        """
        根据 BaseFile 对象下载文件
        :param file: [BaseFile] 文件对象
        :param local_folder: Optional[str] 本地文件夹路径, 默认为当前目录, 即下载到哪里
        :return: [str] 本地文件路径

        用法示例:
        >>> from aligo import Aligo
        >>> ali = Aligo()
        >>> # ll = ali.get_file_list()
        >>> # result = ali.download_file(file=ll[-1])
        >>> result = ali.download_file(file=BaseFile(file_id='<file_id>'))
        >>> print(result)
        """

    def download_file(
            self, *, file_path: str = None, url: str = None,
            local_folder: str = '.', file_id: str = None,
            file: BaseFile = None, drive_id=None
    ) -> str:
        """download_file"""
        if file_id:
            file = self._core_get_file(GetFileRequest(file_id=file_id, drive_id=drive_id))

        if file:
            if file.type == 'folder':
                raise AligoException('文件类型不对：期待文件，得到的是文件夹')
            file_path = os.path.join(local_folder, file.name)
            url = file.download_url or file.url

        return self._core_download_file(file_path, url)



"""..."""
from dataclasses import dataclass

from aligo.types import DatClass


@dataclass
class GetShareTokenResponse(DatClass, str):
    """..."""
    share_token: str = None
    expire_time: str = None
    expires_in: int = None
    share_id: str = None
    share_pwd: str = None

    def __new__(cls, *args, **kwargs):
        share_token = kwargs.get('share_token')
        if share_token is None:
            share_token = args[0]
        DatClass.__new__(cls, *args, **kwargs)
        return str.__new__(cls, share_token)

    def __str__(self):
        return self.share_token

    def __repr__(self):
        return f'{self.share_token!r}'



"""Other"""
from aligo.core import Core
from aligo.core.Config import V2_FILE_GET_OFFICE_PREVIEW_URL
from aligo.request import GetFilePathRequest
from aligo.response import GetFilePathResponse, GetOfficePreviewUrlResponse


class Other(Core):
    """Other"""

    def get_path(self, file_id: str, drive_id: str = None) -> GetFilePathResponse:
        """
        官方：获取文件（夹）目录信息
        :param file_id: 文件（夹）ID
        :param drive_id: 文件（夹）所在网盘ID
        :return: [GetFilePathResponse]

        用法示例：
        >>> from aligo import Aligo, BaseFile, GetFilePathResponse
        >>> aligo = Aligo()
        >>> file_path_info = aligo.get_path('60f927edf4c9f64d3a0c4704b80154cfa3d13c2a')
        >>> assert isinstance(file_path_info, GetFilePathResponse)
        >>> for item in file_path_info.items:
        >>>     assert isinstance(item, BaseFile)
        """
        body = GetFilePathRequest(file_id=file_id, drive_id=drive_id)
        return self._core_get_path(body)

    def get_office_preview_url(self, file_id: str, drive_id: str = None):
        response = self.post(V2_FILE_GET_OFFICE_PREVIEW_URL, body={
            'file_id': file_id,
            'drive_id': drive_id
        })
        return self._result(response, GetOfficePreviewUrlResponse)



"""..."""
from aligo.core.Config import V2_ARCHIVE_UNCOMPRESS, V2_ARCHIVE_STATUS
from aligo.request import ArchiveUncompressRequest, ArchiveStatusRequest
from aligo.response import ArchiveUncompressResponse, ArchiveStatusResponse
from .BaseAligo import BaseAligo


class Compress(BaseAligo):
    """..."""

    def _core_archive_uncompress(self, body: ArchiveUncompressRequest) -> ArchiveUncompressResponse:
        """新建在线解压缩任务"""
        if body.target_drive_id is None:
            body.target_drive_id = self.default_drive_id
        response = self.post(V2_ARCHIVE_UNCOMPRESS, body=body)
        return self._result(response, ArchiveUncompressResponse, status_code=202)

    def _core_archive_status(self, body: ArchiveStatusRequest) -> ArchiveStatusResponse:
        """获取在线解压缩任务状态"""
        response = self.post(V2_ARCHIVE_STATUS, body=body)
        return self._result(response, ArchiveStatusResponse)



"""..."""
from dataclasses import dataclass, field
from typing import List

from aligo.types import DatClass
from aligo.types import UploadPartInfo


@dataclass
class CompleteFileRequest(DatClass):
    """..."""
    file_id: str = None
    drive_id: str = None
    upload_id: str = None
    part_info_list: List[UploadPartInfo] = field(default_factory=list, repr=False)



"""收藏相关"""
from typing import Iterator

from aligo.core.Config import V2_FILE_LIST_BY_CUSTOM_INDEX_KEY
from aligo.request import (StarredFileRequest, UpdateFileRequest, BatchStarFilesRequest, BatchRequest, BatchSubRequest,
                           GetStarredListRequest)
from aligo.response import GetStarredListResponse, BatchSubResponse
from aligo.types import BaseFile
from .Update import Update


class Star(Update):
    """..."""
    _MAX_STAR_COUNT: int = 500

    def _core_starred_file(self, body: StarredFileRequest) -> BaseFile:
        """收藏(或取消) 文件"""
        return self.update_file(UpdateFileRequest(**body.to_dict()))

    def _core_batch_star_files(self, body: BatchStarFilesRequest) -> Iterator[BatchSubResponse[BaseFile]]:
        """批量收藏文件"""
        if body.drive_id is None:
            body.drive_id = self.default_drive_id
        if body.starred:
            custom_index_key = 'starred_yes'
        else:
            custom_index_key = ''

        yield from self.batch_request(BatchRequest(
            requests=[BatchSubRequest(
                id=file_id,
                url='/file/update',
                method='PUT',
                body=StarredFileRequest(
                    drive_id=body.drive_id, file_id=file_id,
                    starred=body.starred, custom_index_key=custom_index_key
                )
            ) for file_id in body.file_id_list]
        ), BaseFile)

    def _core_get_starred_list(self, body: GetStarredListRequest) -> Iterator[BaseFile]:
        """收藏(或取消) 文件列表"""
        yield from self._list_file(V2_FILE_LIST_BY_CUSTOM_INDEX_KEY, body, GetStarredListResponse)



"""FaceThumbnail"""
from dataclasses import dataclass

from .Type import DatClass


@dataclass
class FaceThumbnail(DatClass):
    """FaceThumbnail"""
    face_id: str = None
    face_group_id: str = None
    face_thumbnail: str = None



"""..."""
from dataclasses import dataclass
from typing import List

from aligo.types import DatClass


@dataclass
class BatchStarFilesRequest(DatClass):
    """..."""
    drive_id: str = None
    file_id_list: List[str] = None
    starred: bool = True



"""..."""
from .Album import Album
from .Audio import Audio
from .Compress import Compress
from .Copy import Copy
from .Create import Create
from .Download import Download
from .Drive import Drive
from .Duplicate import Duplicate
from .File import File
from .Move import Move
from .Other import Other
from .Recyclebin import Recyclebin
from .SBox import SBox
from .Search import Search
from .Share import Share
from .Star import Star
from .User import User
from .Video import Video


class Core(
    Album,
    Audio,
    Compress,
    Copy,
    Create,
    Download,
    Drive,
    Duplicate,
    File,
    Move,
    Other,
    Recyclebin,
    SBox,
    Search,
    Share,
    Star,
    User,
    Video,
):
    """..."""



"""..."""

from dataclasses import dataclass

from .CroppingBoundary import CroppingBoundary
from .Type import DatClass


@dataclass
class CroppingSuggestionItem(DatClass):
    """..."""
    aspect_ratio: str = None
    cropping_boundary: CroppingBoundary = None
    score: int = None



"""..."""
from dataclasses import dataclass

from .Type import DatClass


@dataclass
class CroppingBoundary(DatClass):
    """..."""
    height: int = None
    left: int = None
    top: int = None
    width: int = None



"""..."""
from dataclasses import dataclass, field
from typing import List

from aligo.types import DatClass


@dataclass
class ArchiveStatusResponse(DatClass):
    """..."""
    file_list: List[str] = field(default_factory=list)
    progress: int = None
    state: str = None
    task_id: str = None



"""..."""
from dataclasses import dataclass
from dataclasses import field
from typing import Dict, List

from .Enum import BaseFileCategory, BaseFileContentHashName, BaseFileType
from .FieldsInfo import FieldsInfo
from .ImageMedia import ImageMedia
from .Type import DatClass
from .VideoMedia import VideoMedia
from .VideoPreview import VideoPreview


@dataclass
class ShareLinkBaseFile(DatClass):
    """..."""
    type: BaseFileType = None
    file_id: str = None
    name: str = None
    parent_file_id: str = field(default=None, repr=False)
    category: BaseFileCategory = field(default=None, repr=False)
    size: int = field(default=None, repr=False)
    created_at: str = field(default=None, repr=False)
    content_type: str = field(default=None, repr=False)
    description: str = field(default=None, repr=False)
    content_hash: str = field(default=None, repr=False)
    content_hash_name: BaseFileContentHashName = field(default=None, repr=False)
    crc64_hash: str = field(default=None, repr=False)
    domain_id: str = field(default=None, repr=False)
    download_url: str = field(default=None, repr=False)
    drive_id: str = field(default=None, repr=False)
    encrypt_mode: str = field(default=None, repr=False)
    file_extension: str = field(default=None, repr=False)
    hidden: bool = field(default=None, repr=False)
    image_media_metadata: ImageMedia = field(default=None, repr=False)
    labels: list = field(default=None, repr=False)
    meta: str = field(default=None, repr=False)
    mime_extension: str = field(default=None, repr=False)
    mime_type: str = field(default=None, repr=False)
    punish_flag: int = field(default=None, repr=False)
    starred: bool = field(default=None, repr=False)
    status: str = field(default=None, repr=False)
    streams_url_info: Dict = field(default=None, repr=False)
    streams_info: Dict = field(default=None, repr=False)
    thumbnail: str = field(default=None, repr=False)
    trashed: bool = field(default=None, repr=False)
    trashed_at: str = field(default=None, repr=False)
    updated_at: str = field(default=None, repr=False)
    upload_id: str = field(default=None, repr=False)
    url: str = field(default=None, repr=False)
    user_meta: str = field(default=None, repr=False)
    video_media_metadata: VideoMedia = field(default=None, repr=False)
    video_preview_metadata: VideoPreview = field(default=None, repr=False)
    location: str = field(default=None, repr=False)
    action_list: List[str] = field(default_factory=list, repr=False)
    user_tags: Dict = field(default=None, repr=False)
    last_modifier_type: str = field(default=None, repr=False)
    last_modifier_id: str = field(default=None, repr=False)
    last_modifier_name: str = field(default=None, repr=False)
    creator_type: str = field(default=None, repr=False)
    creator_id: str = field(default=None, repr=False)
    creator_name: str = field(default=None, repr=False)
    revision_id: str = field(default=None, repr=False)
    sync_flag: bool = field(default=None, repr=False)
    sync_device_flag: bool = field(default=None, repr=False)
    sync_meta: str = field(default=None, repr=False)
    ex_fields_info: FieldsInfo = field(default=None, repr=False)
    from_share_id: str = None
    revision_version: int = None
    channel: str = None
    meta_name_punish_flag: int = None
    meta_name_investigation_status: int = None



"""..."""
from dataclasses import dataclass

from aligo.types import DatClass
from aligo.types.Enum import CheckNameMode


@dataclass
class RenameFileRequest(DatClass):
    """..."""
    name: str
    file_id: str
    check_name_mode: CheckNameMode = 'refuse'
    drive_id: str = None



"""扫描二维码登录服务，供登录使用"""
from http.server import BaseHTTPRequestHandler
from typing import Any


class LoginServer(BaseHTTPRequestHandler):
    """..."""

    # noinspection PyPep8Naming
    def do_GET(self):
        """..."""
        if self.path == '/':
            self.send_response(200)
            self.send_header("content-type", "text/html")
            self.end_headers()
            self.wfile.write(bytes(
                f"""<!DOCTYPE html>
<html lang="en">
<head>
    <meta http-equiv="Content-Type" content="text/html;charset=utf-8">
    <title>扫码登录阿里云盘</title>
</head>
<body>
<p align="center"><img src="/login.png"></p>
</body>
</html>""", 'utf8'))
        elif self.path == '/login.png':
            self.send_response(200)
            self.send_header("content-type", "image/png")
            self.end_headers()
            # noinspection PyUnresolvedReferences
            self.wfile.write(self.server.qrData)
        elif self.path == '/close':
            self.server.server_close()
        else:
            self.send_response(404)

    def log_message(self, _format: str, *args: Any) -> None:
        """..."""
        return



"""..."""
from dataclasses import dataclass

from .Type import DatClass


@dataclass
class LoginDevice(DatClass):
    """..."""
    deviceId: str = None
    deviceName: str = None
    modelName: str = None
    city: str = None
    login_time: str = None



"""..."""
from dataclasses import dataclass

from aligo.types import DatClass
from aligo.types.Enum import GetShareLinkListOrderBy, OrderDirection


@dataclass
class AlbumListRequest(DatClass):
    """..."""
    album_drive_id: str = None
    drive_id: str = None
    order_by: GetShareLinkListOrderBy = 'created_at'
    order_direction: OrderDirection = 'DESC'



class AligoException(Exception):
    pass


class AligoFatalError(AligoException):
    pass


class AligoStatus500(AligoException):
    pass


class AligoRefreshFailed(AligoException):
    pass


class AligoShareLinkCreateExceedDailyLimit(AligoException):
    pass



"""..."""
from dataclasses import dataclass

from aligo.types import DatClass


@dataclass
class GetOfficePreviewUrlResponse(DatClass):
    """..."""
    access_token: str = None
    preview_url: str = None



"""..."""
from dataclasses import dataclass, field
from typing import List

from aligo.types import DatClass


@dataclass
class VipInfo(DatClass):
    name: str = None
    code: str = None
    promotedAt: int = None
    expire: int = None


@dataclass
class UsersVipInfoResponse(DatClass):
    """..."""
    identity: str = None
    icon: str = None
    mediumIcon: str = None
    status: str = None
    vipList: List[VipInfo] = field(default_factory=list)



"""..."""
import itertools
from typing import List

from aligo.core import Core
from aligo.request import ListToCleanRequest
from aligo.response import DuplicateItem
from aligo.types import BaseFile


class Duplicate(Core):
    """..."""

    def duplicate_list(self, drive_id: str = None) -> List[DuplicateItem]:
        """..."""
        return list(self._core_duplicate_list(drive_id))

    def list_to_clean(self, album_drive_id: str, size: int = 200, drive_id: str = None) -> List[BaseFile]:
        """..."""
        if drive_id is None:
            drive_id = self.default_drive_id
        ll = []
        for i in itertools.islice(
                self._core_list_to_clean(ListToCleanRequest(drive_id=drive_id, album_drive_id=album_drive_id)), size):
            ll.append(i)
        return ll



"""..."""
from dataclasses import dataclass, field
from typing import List

from aligo.types import DatClass, AudioTranscodeTemplate, VideoTranscodeTemplate


@dataclass
class GetShareLinkDownloadUrlResponse(DatClass):
    """..."""
    download_url: str = None
    url: str = None
    thumbnail: str = None
    video_template_list: List[VideoTranscodeTemplate] = field(default_factory=list)
    audio_template_list: List[AudioTranscodeTemplate] = field(default_factory=list)



"""..."""
from dataclasses import dataclass
from typing import List

from aligo.core import BaseAligo
from aligo.core.Config import (V2_DRIVE_GET, V2_DRIVE_GET_DEFAULT_DRIVE, V2_DRIVE_LIST_MY_DRIVES,
                               ADRIVE_V1_USER_GET_USER_CAPACITY_INFO)
from aligo.request import GetDriveRequest, GetDefaultDriveRequest
from aligo.response import ListMyDrivesResponse
from aligo.types import BaseDrive
from aligo.types.Type import DatClass


@dataclass
class UserCapacityLimitDetails(DatClass):
    limit_consume: bool = None
    limit_download: bool = None


@dataclass
class CapacityLevelInfo(DatClass):
    capacity_type: str = None


@dataclass
class DriveCapacityDetails(DatClass):
    album_drive_used_size: int = None
    backup_drive_used_size: int = None
    default_drive_used_size: int = None
    drive_total_size: int = None
    drive_used_size: int = None
    note_drive_used_size: int = None
    resource_drive_used_size: int = None
    sbox_drive_used_size: int = None
    share_album_drive_used_size: int = None


@dataclass
class UserCapacityInfo(DatClass):
    capacity_level_info: CapacityLevelInfo = None
    drive_capacity_details: DriveCapacityDetails = None
    user_capacity_limit_details: UserCapacityLimitDetails = None


class Drive(BaseAligo):
    """..."""

    def _core_get_drive(self, body: GetDriveRequest) -> BaseDrive:
        """..."""
        response = self.post(V2_DRIVE_GET, body=body)
        return self._result(response, BaseDrive)

    def get_default_drive(self) -> BaseDrive:
        """
        Get default drive.
        :return: [BaseDrive]

        :Example:
        >>> from aligo import Aligo
        >>> ali = Aligo()
        >>> drive = ali.get_default_drive()
        >>> print(drive.drive_name)
        """
        if self._default_drive is None:
            response = self.post(V2_DRIVE_GET_DEFAULT_DRIVE, body=GetDefaultDriveRequest(self._auth.token.user_id))
            self._default_drive = self._result(response, BaseDrive)
        return self._default_drive

    def list_my_drives(self) -> List[BaseDrive]:
        """
        List my drives.
        :return: [BaseDrive]

        :Example:
        >>> from aligo import Aligo
        >>> ali = Aligo()
        >>> drives = ali.list_my_drives()
        >>> for drive in drives:
        >>>     print(drive.drive_name)
        """
        # noinspection PyTypeChecker
        return list(self._list_file(V2_DRIVE_LIST_MY_DRIVES, {}, ListMyDrivesResponse))

    def get_user_capacity_info(self) -> UserCapacityInfo:
        response = self.post(ADRIVE_V1_USER_GET_USER_CAPACITY_INFO)
        return self._result(response, UserCapacityInfo)



from dataclasses import dataclass

from aligo.core import BaseAligo
from aligo.core.Config import V2_SBOX_GET
from aligo.types.Type import DatClass


@dataclass
class SBoxResponse(DatClass):
    drive_id: str = None
    insurance_enabled: bool = None
    locked: bool = None
    pin_setup: bool = None
    recommend_vip: str = None
    sbox_real_used_size: int = None
    sbox_total_size: int = None
    sbox_used_size: int = None


class SBox(BaseAligo):

    def v2_sbox_get(self) -> SBoxResponse:
        response = self.post(V2_SBOX_GET)
        return self._result(response, SBoxResponse)



"""..."""
from aligo.core import Core
from aligo.request import GetVideoPlayInfoRequest, GetVideoPreviewPlayInfoRequest
from aligo.response import GetVideoPlayInfoResponse, GetVideoPreviewPlayInfoResponse
from aligo.types.Enum import VideoTemplateID


class Video(Core):
    """..."""

    def get_video_play_info(self, file_id: str, drive_id: str = None) -> GetVideoPlayInfoResponse:
        """
        获取视频播放信息
        :param file_id: [必须] 视频文件ID
        :param drive_id: [可选] 文件所在的网盘盘符ID
        :return: [GetVideoPlayInfoResponse]

        :Example:
        >>> from aligo import Aligo
        >>> ali = Aligo()
        >>> video = ali.get_video_play_info('<file_id>')
        >>> print(video)
        """
        body = GetVideoPlayInfoRequest(file_id=file_id, drive_id=drive_id)
        return self._core_get_video_play_info(body)

    def get_video_preview_play_info(self, file_id: str, template_id: VideoTemplateID = '',
                                    drive_id: str = None) -> GetVideoPreviewPlayInfoResponse:
        """
        获取视频预览播放信息
        :param file_id: [必须] 视频文件ID
        :param template_id: [可选] 视频模板ID
        :param drive_id: [可选] 文件所在的网盘盘符ID
        :return: [GetVideoPreviewPlayInfoResponse]

        :Example:
        >>> from aligo import Aligo
        >>> ali = Aligo()
        >>> video = ali.get_video_preview_play_info('<file_id>')
        >>> print(video)
        """
        body = GetVideoPreviewPlayInfoRequest(file_id=file_id, template_id=template_id, drive_id=drive_id)
        return self._core_get_video_preview_play_info(body)



"""..."""
from dataclasses import dataclass, field
from typing import List

from aligo.types import DatClass, DriveFile


@dataclass
class PrivateShareRequest(DatClass):
    drive_file_list: List[DriveFile] = field(default_factory=list)



"""..."""
from typing import Iterator

from aligo.core import BaseAligo
from aligo.core.Config import V2_FILE_SEARCH, V2_AIMS_SEARCH
from aligo.request import SearchFileRequest, AimSearchRequest
from aligo.response import SearchFileResponse, AimSearchResponse
from aligo.types import BaseFile


class Search(BaseAligo):
    """..."""

    def _core_search_files(self, body: SearchFileRequest) -> Iterator[BaseFile]:
        """
        关于 query 的语法, 参考下段代码
        {
            key: "getPDSSearchQuery", value: function () {
                var n = ['name match "'.concat(this.queryToSearch, '"')];
                return this.filter && ("folder" === this.filter ? n.push('type = "'.concat(this.filter, '"')) : n.push('category = "'.concat(this.filter, '"'))), n.join(" and ")
            }
        }
        eg: 'name match "epub"'
        eg: 'name match "epub" and category = "image"'
        category : BaseFileCategory
        """
        yield from self._list_file(V2_FILE_SEARCH, body, SearchFileResponse)

    def _core_search_aims(self, body: AimSearchRequest) -> Iterator[BaseFile]:
        """
        {
          "drive_id": "1067819",
          "limit": 20,
          "marker": "",
          "order_by": "image_time DESC,last_access_at DESC,updated_at DESC",
          "query": "keywords ='画画' and type = 'file' and category = 'image' and status = 'available' and hidden = false and status = 'available' and hidden = false",
          "return_total_count": true
        }
        """
        yield from self._list_file(V2_AIMS_SEARCH, body, AimSearchResponse)



"""..."""
from dataclasses import dataclass, field
from typing import List

from aligo.types import DatClass, ShareLinkBaseFile
from aligo.types.Enum import SharePolicy


@dataclass
class CreateShareLinkResponse(DatClass):
    """
    分享链接结构对象, 类似 BaseShareFile 的含义
    Attributes:
        created_at (str): 分享创建时间
        updated_at (str): 分享更新时间
        creator (str): 分享创建者
        description (str): 描述
        download_count (int):
        preview_count (int):
        save_count (int):
        drive_id (str):
        expiration (str): 分享过期时间
        expired (str): 分享是否过期
        file_id (str): 如果create时候是单个文件，那么file_id字段也有效，表示分享的一个文件的 id
        file_id_list (List[str]): 分享的一组文件或者文件加的列表
        share_id (str):
        share_msg (str): 分享口令
        share_name (str): 分享名
        share_policy (SharePolicy):
        share_pwd (str):
        share_url (str):
        status (str):
        first_file (ShareLinkBaseFile): 分享中首个文件的信息
    """
    created_at: str = None
    creator: str = None
    description: str = None
    download_count: int = 0
    drive_id: str = None
    expiration: str = None
    expired: bool = False
    file_id: str = None
    file_id_list: List[str] = field(default_factory=list)
    preview_count: int = 0
    save_count: int = 0
    share_id: str = None
    share_msg: str = None
    share_name: str = None
    share_policy: SharePolicy = None
    share_pwd: str = ''
    share_url: str = None
    status: str = None
    updated_at: str = None
    popularity: int = None
    is_photo_collection: bool = False
    sync_to_homepage: bool = False
    full_share_msg: str = None
    popularity_str: str = None
    display_name: str = None
    first_file: ShareLinkBaseFile = None
